# https://pubs.opengroup.org/onlinepubs/009695399/basedefs/sys/resource.h.html

from posix.time  cimport timeval
from posix.types cimport id_t

cdef extern from "<sys/resource.h>" nogil:

    enum: PRIO_PROCESS
    enum: PRIO_PGRP
    enum: PRIO_USER

    enum: RLIM_INFINITY
    enum: RLIM_SAVED_MAX
    enum: RLIM_SAVED_CUR

    enum: RUSAGE_SELF
    enum: RUSAGE_CHILDREN

    enum: RLIMIT_CORE
    enum: RLIMIT_CPU
    enum: RLIMIT_DATA
    enum: RLIMIT_FSIZE
    enum: RLIMIT_NOFILE
    enum: RLIMIT_STACK
    enum: RLIMIT_AS

    ctypedef unsigned long rlim_t

    cdef struct rlimit:
        rlim_t rlim_cur
        rlim_t rlim_max

    cdef struct rusage:
        timeval ru_utime
        timeval ru_stime
        long    ru_maxrss
        long    ru_ixrss
        long    ru_idrss
        long    ru_isrss
        long    ru_minflt
        long    ru_majflt
        long    ru_nswap
        long    ru_inblock
        long    ru_oublock
        long    ru_msgsnd
        long    ru_msgrcv
        long    ru_nsignals
        long    ru_nvcsw
        long    ru_nivcsw

    int  getpriority(int, id_t)
    int  getrlimit(int, rlimit *)
    int  getrusage(int, rusage *)
    int  setpriority(int, id_t, int)
    int  setrlimit(int, const rlimit *)



import os
import pytest

import numpy as np

from tifffile import tifffile
from brainio import brainio

from imlib.cells.cells import Cell
from imlib.IO.cells import get_cells

from imlib.general.system import (
    delete_directory_contents,
    get_sorted_file_paths,
)
import cellfinder.extract.extract_cubes as extract_cubes

data_dir = os.path.join("tests", "data")

signal_data_dir = os.path.join(data_dir, "signal")
background_data_dir = os.path.join(data_dir, "background")
xml_path = os.path.join(data_dir, "cube_extract", "cells.xml")
validate_cubes_dir = os.path.join(data_dir, "cube_extract", "cubes")
validate_cubes_scale_dir = os.path.join(
    data_dir, "cube_extract", "cubes_scale"
)


class CubeExtractArgs:
    def __init__(self, tmpdir):
        self.cells_file_path = xml_path

        self.cube_width = 50
        self.cube_height = 50
        self.cube_depth = 20
        self.save_empty_cubes = False

        self.x_pixel_um = 1
        self.y_pixel_um = 1
        self.z_pixel_um = 5

        # get these from parser defaults
        self.x_pixel_um_network = 1
        self.y_pixel_um_network = 1
        self.z_pixel_um_network = 5

        self.n_free_cpus = 0
        self.n_max_threads = 10
        self.max_ram = None

        self.paths = Paths(cells_file_path=xml_path, cubes_output_dir=tmpdir)

    @staticmethod
    def get_plane_paths():
        return signal_data_dir + background_data_dir


class Paths:
    def __init__(self, cells_file_path=None, cubes_output_dir=None):
        self.cells_file_path = cells_file_path
        self.tmp__cubes_output_dir = cubes_output_dir


def load_cubes_in_dir(directory):
    cube_list = os.listdir(directory)
    cubes = []
    for file in cube_list:
        file_path = os.path.join(directory, file)
        cubes.append(brainio.load_any(file_path))
    return cubes


def test_cube_extraction(tmpdir, depth=20):
    tmpdir = str(tmpdir)
    args = CubeExtractArgs(tmpdir)

    planes_paths = {}
    planes_paths[0] = get_sorted_file_paths(
        signal_data_dir, file_extension="tif"
    )
    planes_paths[1] = get_sorted_file_paths(
        background_data_dir, file_extension="tif"
    )

    extract_cubes.main(
        get_cells(args.paths.cells_file_path),
        args.paths.tmp__cubes_output_dir,
        planes_paths,
        args.cube_depth,
        args.cube_width,
        args.cube_height,
        args.x_pixel_um,
        args.y_pixel_um,
        args.z_pixel_um,
        args.x_pixel_um_network,
        args.y_pixel_um_network,
        args.z_pixel_um_network,
        args.max_ram,
        args.n_free_cpus,
        args.save_empty_cubes,
    )

    validation_cubes = load_cubes_in_dir(validate_cubes_dir)
    test_cubes = load_cubes_in_dir(tmpdir)

    for idx, test_cube in enumerate(test_cubes):
        assert (validation_cubes[idx] == test_cube).all()

    delete_directory_contents(tmpdir)

    # test cube scaling
    args.x_pixel_um = 2
    args.y_pixel_um = 2
    args.z_pixel_um = 7.25

    extract_cubes.main(
        get_cells(args.paths.cells_file_path),
        args.paths.tmp__cubes_output_dir,
        planes_paths,
        args.cube_depth,
        args.cube_width,
        args.cube_height,
        args.x_pixel_um,
        args.y_pixel_um,
        args.z_pixel_um,
        args.x_pixel_um_network,
        args.y_pixel_um_network,
        args.z_pixel_um_network,
        args.max_ram,
        args.n_free_cpus,
        args.save_empty_cubes,
    )

    validation_cubes_scale = load_cubes_in_dir(validate_cubes_scale_dir)
    test_cubes = load_cubes_in_dir(tmpdir)
    for idx, test_cube in enumerate(test_cubes):
        assert (validation_cubes_scale[idx] == test_cube).all()

    #  test edge of data errors
    cell = Cell("x0y0z10", 2)
    plane_paths = os.listdir(signal_data_dir)
    first_plane = tifffile.imread(
        os.path.join(signal_data_dir, plane_paths[0])
    )
    stack_shape = first_plane.shape + (depth,)
    stacks = {}
    stacks[0] = np.zeros(stack_shape, dtype=np.uint16)
    stacks[0][:, :, 0] = first_plane

    for plane in range(1, depth):
        im_path = os.path.join(signal_data_dir, plane_paths[plane])
        stacks[0][:, :, plane] = tifffile.imread(im_path)

    cube = extract_cubes.Cube(cell, 0, stacks)
    assert (cube.data == 0).all()

    cell = Cell("x2500y2500z10", 2)
    cube = extract_cubes.Cube(cell, 0, stacks)
    assert (cube.data == 0).all()

    # test insufficient z-planes for a specific cube
    stacks[0] = stacks[0][:, :, 1:]
    cube = extract_cubes.Cube(cell, 0, stacks)
    assert (cube.data == 0).all()

    # test insufficient z-planes for any cube to be extracted at all.
    delete_directory_contents(tmpdir)
    args.z_pixel_um = 0.1

    with pytest.raises(extract_cubes.StackSizeError):

        extract_cubes.main(
            get_cells(args.paths.cells_file_path),
            args.paths.tmp__cubes_output_dir,
            planes_paths,
            args.cube_depth,
            args.cube_width,
            args.cube_height,
            args.x_pixel_um,
            args.y_pixel_um,
            args.z_pixel_um,
            args.x_pixel_um_network,
            args.y_pixel_um_network,
            args.z_pixel_um_network,
            args.max_ram,
            args.n_free_cpus,
            args.save_empty_cubes,
        )



from __future__ import absolute_import

import cython

from ..Plex.Scanners cimport Scanner

cdef unicode any_string_prefix, IDENT

cdef get_lexicon()
cdef initial_compile_time_env()

## methods commented with '##' out are used by Parsing.py when compiled.

@cython.final
cdef class CompileTimeScope:
    cdef public dict entries
    cdef public CompileTimeScope outer
    ##cdef declare(self, name, value)
    ##cdef lookup_here(self, name)
    ##cpdef lookup(self, name)

@cython.final
cdef class PyrexScanner(Scanner):
    cdef public context
    cdef public list included_files
    cdef public CompileTimeScope compile_time_env
    cdef public bint compile_time_eval
    cdef public bint compile_time_expr
    cdef public bint parse_comments
    cdef public bint in_python_file
    cdef public source_encoding
    cdef set keywords
    cdef public list indentation_stack
    cdef public indentation_char
    cdef public int bracket_nesting_level
    cdef readonly bint async_enabled
    cdef public sy
    cdef public systring

    cdef long current_level(self)
    #cpdef commentline(self, text)
    #cpdef open_bracket_action(self, text)
    #cpdef close_bracket_action(self, text)
    #cpdef newline_action(self, text)
    #cpdef begin_string_action(self, text)
    #cpdef end_string_action(self, text)
    #cpdef unclosed_string_action(self, text)
    @cython.locals(current_level=cython.long, new_level=cython.long)
    cpdef indentation_action(self, text)
    #cpdef eof_action(self, text)
    ##cdef next(self)
    ##cdef peek(self)
    #cpdef put_back(self, sy, systring)
    #cdef unread(self, token, value)
    ##cdef bint expect(self, what, message = *) except -2
    ##cdef expect_keyword(self, what, message = *)
    ##cdef expected(self, what, message = *)
    ##cdef expect_indent(self)
    ##cdef expect_dedent(self)
    ##cdef expect_newline(self, message=*, bint ignore_semicolon=*)
    ##cdef int enter_async(self) except -1
    ##cdef int exit_async(self) except -1



from libc.stdio cimport FILE

cdef extern from "Python.h":

    ###########################################################################
    # Data marshalling support
    ###########################################################################

    const int Py_MARSHAL_VERSION

    void PyMarshal_WriteLongToFile(long value, FILE *file, int version)
    # Marshal a long integer, value, to file. This will only write the
    # least-significant 32 bits of value, regardless of the size of the native
    # long type. version indicates the file format.

    void PyMarshal_WriteObjectToFile(object value, FILE *file, int version)
    # Marshal a Python object, value, to file. version indicates the file
    # format.

    bytes PyMarshal_WriteObjectToString(object value, int version)
    # Return value: New reference.
    # Return a bytes object containing the marshalled representation of value.
    # version indicates the file format.

    long PyMarshal_ReadLongFromFile(FILE *file) except? -1
    # Return a C long from the data stream in a FILE* opened for reading. Only
    # a 32-bit value can be read in using this function, regardless of the
    # native size of long.

    # On error, sets the appropriate exception (EOFError) and returns -1.

    int PyMarshal_ReadShortFromFile(FILE *file) except? -1
    # Return a C short from the data stream in a FILE* opened for reading. Only
    # a 16-bit value can be read in using this function, regardless of the
    # native size of short.

    # On error, sets the appropriate exception (EOFError) and returns -1.

    object PyMarshal_ReadObjectFromFile(FILE *file)
    # Return value: New reference.
    # Return a Python object from the data stream in a FILE* opened for
    # reading.
    
    # On error, sets the appropriate exception (EOFError, ValueError or
    # TypeError) and returns NULL.

    object PyMarshal_ReadLastObjectFromFile(FILE *file)
    # Return value: New reference.
    # Return a Python object from the data stream in a FILE* opened for
    # reading. Unlike PyMarshal_ReadObjectFromFile(), this function assumes
    # that no further objects will be read from the file, allowing it to
    # aggressively load file data into memory so that the de-serialization can
    # operate from data in memory, rather than reading a byte at a time from the
    # file. Only use these variant if you are certain that you won’t be reading
    # anything else from the file.

    # On error, sets the appropriate exception (EOFError, ValueError or
    # TypeError) and returns NULL.

    object PyMarshal_ReadObjectFromString(const char *data, Py_ssize_t len)
    # Return value: New reference.
    # Return a Python object from the data stream in a byte buffer containing
    # len bytes pointed to by data.

    # On error, sets the appropriate exception (EOFError, ValueError or
    # TypeError) and returns NULL.



import os
from imlib.general.system import ensure_directory_exists

from cellfinder.download.download import download

default_atlas_name = "brain.nii"


model_weight_urls = {
    "resnet50_all": "https://gin.g-node.org/cellfinder/models/raw/master/resnet50_weights.h5",
    "resnet50_tv": "https://gin.g-node.org/cellfinder/models/raw/master/resnet50_tv.h5",
}

download_requirements_gb = {
    "resnet50_all": 0.18,
    "resnet50_tv": 0.18,
}


def main(model, download_path):
    model_weight_dir = os.path.join(download_path, "model_weights")
    model_path = os.path.join(model_weight_dir, model + ".h5")
    if not os.path.exists(model_path):
        ensure_directory_exists(model_weight_dir)

        download_path = os.path.join(model_weight_dir, model + ".h5")
        download(
            download_path,
            model_weight_urls[model],
            model,
            download_requires=download_requirements_gb[model],
        )

    else:
        print(f"Model already exists at {model_path}. Skipping download")

    return model_path



import math

import numpy as np

from cellfinder.detect.filters.plane_filters.base_tile_filter import (
    OutOfBrainTileFilter,
)


class TileWalker(object):
    def __init__(self, img, soma_diameter):
        self.img = img
        self.thresholded_img = img.copy()
        self.img_width, self.img_height = img.shape
        self.soma_diameter = soma_diameter
        self.tile_width = self.soma_diameter * 2
        self.tile_height = self.soma_diameter * 2

        n_tiles_width = math.ceil(self.img_width / self.tile_width)
        n_tiles_height = math.ceil(self.img_height / self.tile_height)
        self.good_tiles_mask = np.zeros(
            (n_tiles_width, n_tiles_height), dtype=np.bool
        )

        self.x = 0
        self.y = 0
        self.tile_idx = 0

        corner_intensity = img[
            0 : self.tile_width, 0 : self.tile_height
        ].mean()
        corner_sd = img[0 : self.tile_width, 0 : self.tile_height].std()
        out_of_brain_threshold = corner_intensity + (2 * corner_sd)

        self.ftf = OutOfBrainTileFilter(
            out_of_brain_intensity_threshold=out_of_brain_threshold
        )

    def _get_tiles(self):  # WARNING: crops to integer steps
        for y in range(
            0, self.img_height - self.tile_height, self.tile_height
        ):
            for x in range(
                0, self.img_width - self.tile_width, self.tile_width
            ):
                read_tile = self.img[
                    x : x + self.tile_width, y : y + self.tile_height
                ]
                write_tile = self.thresholded_img[
                    x : x + self.tile_width, y : y + self.tile_height
                ]
                yield x, y, read_tile, write_tile

    def walk_out_of_brain_only(self):
        for x, y, tile, write_tile in self._get_tiles():
            self.x = x
            self.y = y
            self.ftf.set_tile(tile)
            if self.ftf.out_of_brain_intensity_threshold:
                if not self.ftf.is_low_average():
                    mask_x = self.x // self.tile_width
                    mask_y = self.y // self.tile_height
                    self.good_tiles_mask[mask_x, mask_y] = True



cdef extern from "<limits>" namespace "std" nogil:
   enum float_round_style:
        round_indeterminate       = -1
        round_toward_zero         = 0
        round_to_nearest          = 1
        round_toward_infinity     = 2
        round_toward_neg_infinity = 3

   enum float_denorm_style:
        denorm_indeterminate  = -1
        denorm_absent         = 0
        denorm_present        = 1

   #The static methods can be called as, e.g. numeric_limits[int].round_error(), etc.
   #The const data members should be declared as static.  Cython currently doesn't allow that
   #and/or I can't figure it out, so you must instantiate an object to access, e.g.
   #cdef numeric_limits[double] lm
   #print lm.round_style
   cdef cppclass numeric_limits[T]:
    const bint is_specialized
    @staticmethod
    T min()
    @staticmethod
    T max()
    const int digits
    const int  digits10
    const bint is_signed
    const bint is_integer
    const bint is_exact
    const int radix
    @staticmethod
    T epsilon()
    @staticmethod
    T round_error()

    const int  min_exponent
    const int  min_exponent10
    const int  max_exponent
    const int  max_exponent10

    const bint has_infinity
    const bint has_quiet_NaN
    const bint has_signaling_NaN
    const float_denorm_style has_denorm
    const bint has_denorm_loss
    @staticmethod
    T infinity()
    @staticmethod
    T quiet_NaN()
    @staticmethod
    T signaling_NaN()
    @staticmethod
    T denorm_min()

    const bint is_iec559
    const bint is_bounded
    const bint is_modulo

    const bint traps
    const bint tinyness_before
    const float_round_style round_style



import os
import argparse

import pandas as pd
from imlib.general.list import remove_empty_string, unique_elements_lists

from imlib.source.source_files import get_structures_path


def structure_name_to_id(structure_name, reference_structures_table):
    row = reference_structures_table[
        reference_structures_table["name"] == structure_name
    ]
    return row["structure_id_path"].iloc[0]


def get_substructures(structure_name, reference_structures_table):
    structure_id = structure_name_to_id(
        structure_name, reference_structures_table
    )
    return reference_structures_table[
        reference_structures_table["structure_id_path"].str.startswith(
            structure_id
        )
    ]


def get_sub_results(
    result, structure_name, reference_structures_table, sum_regions=False
):
    substructures = get_substructures(
        structure_name, reference_structures_table
    )
    substructures_names = substructures["name"]
    sub_results = result[result["structure_name"].isin(substructures_names)]
    if sum_regions:
        sub_results = sub_results.sum().to_frame()
        sub_results = sub_results.T
        sub_results["structure_name"] = structure_name
        sub_results["left_cells_per_mm3"] = (
            sub_results["left_cell_count"] / sub_results["left_volume_mm3"]
        )
        sub_results["right_cells_per_mm3"] = (
            sub_results["right_cell_count"] / sub_results["right_volume_mm3"]
        )

    return sub_results


def make_summary_df_for_brain(
    brain_results_path, regions, reference_structures_table, sum_regions=False
):
    result = pd.read_csv(brain_results_path)
    df = pd.DataFrame()
    for region in regions:
        try:
            sub_results = get_sub_results(
                result,
                region,
                reference_structures_table,
                sum_regions=sum_regions,
            )
            df = df.append(sub_results)
        except IndexError:
            print(
                "Brain region: {} could not be found in the atlas "
                "definition. Skipping.".format(region)
            )
    return df


def summary_run(args):
    # if args.structures_file_path is None:
    #     args.structures_file_path = get_structures_path()
    #     reference_structures_table = pd.read_csv(args.structures_file_path)
    # else:
    #     raise NotImplementedError(
    #         "Only the Allen adult mouse atlas is " "currently supported."
    #     )
    reference_structures_table = pd.read_csv(get_structures_path())

    if not args.regions and not args.regions_list:
        # regions = list(pd.read_csv(args.structures_file_path)["name"])
        regions = list(pd.read_csv(get_structures_path())["name"])

    else:
        regions = []
        if args.regions:
            regions = regions + args.regions
        if args.regions_list:
            regions = regions + list(pd.read_csv(args.regions_list)["name"])

    regions = remove_empty_string(regions)
    regions = unique_elements_lists(regions)
    csvs_folder = args.csv_dir
    destination_folder = os.path.join(args.csv_dir, "summary")

    csvs_names = [f for f in os.listdir(csvs_folder) if f.endswith(".csv")]
    if len(csvs_names) == 0:
        raise FileNotFoundError(
            "No CSV files were found in the directory: "
            "{}. Please check the arguments".format(csvs_folder)
        )

    csvs_paths = [os.path.join(csvs_folder, f) for f in csvs_names]

    if not os.path.exists(destination_folder):
        os.makedirs(destination_folder)

    for csv_file_path in csvs_paths:
        print("CSV file: {}".format(os.path.basename(csv_file_path)))
        summary = make_summary_df_for_brain(
            csv_file_path,
            regions,
            reference_structures_table,
            sum_regions=args.sum_regions,
        )
        filename = os.path.basename(csv_file_path)
        dest_path = os.path.join(destination_folder, filename)

        summary.to_csv(dest_path, index=False)
    print("Done!")


def get_parser():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
    )
    parser.add_argument(
        dest="csv_dir",
        type=str,
        help="Directory containing csv files to be summarised",
    )

    parser.add_argument(
        "--regions-list",
        dest="regions_list",
        type=str,
        help="Curated structure list in a text file (as per the allen brain "
        "atlas csv)",
    )
    parser.add_argument(
        "--regions",
        dest="regions",
        nargs="+",
        help="A list of additional regions to include",
    )
    # parser.add_argument(
    #     "--structures-file",
    #     dest="structures_file_path",
    #     type=str,
    #     help="The csv file containing the structures "
    #     "definition (if not using the default "
    #     "Allen brain atlas).",
    # )
    parser.add_argument(
        "--sum-regions",
        dest="sum_regions",
        action="store_true",
        help="Rather than listing substructures, sum them.",
    )
    return parser


def main():
    args = get_parser().parse_args()
    summary_run(args)


if __name__ == "__main__":
    main()



from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
import napari
from napari.utils.io import magic_imread
from imlib.general.system import get_sorted_file_paths

from imlib.IO.cells import cells_xml_to_df
from imlib.cells.cells import Cell


def parser():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    parser.add_argument(dest="img_paths", type=str, help="Directory of images")
    parser.add_argument(
        dest="cells_xml", type=str, help="Path to the .xml cell file"
    )
    parser.add_argument(
        "--symbol", type=str, default="ring", help="Marker symbol."
    )
    parser.add_argument(
        "--marker-size", type=int, default=15, help="Marker size."
    )
    parser.add_argument(
        "--opacity", type=float, default=0.6, help="Opacity of the markers."
    )
    return parser


def cells_df_as_np(cells_df, new_order=[2, 1, 0], type_column="type"):
    cells_df = cells_df.drop(columns=[type_column])
    cells = cells_df[cells_df.columns[new_order]]
    cells = cells.to_numpy()
    return cells


def get_cell_arrays(cells_file):
    df = cells_xml_to_df(cells_file)

    non_cells = df[df["type"] == Cell.UNKNOWN]
    cells = df[df["type"] == Cell.CELL]

    cells = cells_df_as_np(cells)
    non_cells = cells_df_as_np(non_cells)
    return cells, non_cells


def main():
    args = parser().parse_args()
    img_paths = get_sorted_file_paths(args.img_paths, file_extension=".tif")
    cells, non_cells = get_cell_arrays(args.cells_xml)

    with napari.gui_qt():
        v = napari.Viewer(title="Cellfinder cell viewer")
        images = magic_imread(img_paths, use_dask=True, stack=True)
        v.add_image(images)

        v.add_points(
            non_cells,
            size=args.marker_size,
            n_dimensional=True,
            opacity=args.opacity,
            symbol=args.symbol,
            face_color="lightskyblue",
            name="Non-Cells",
        )
        v.add_points(
            cells,
            size=args.marker_size,
            n_dimensional=True,
            opacity=args.opacity,
            symbol=args.symbol,
            face_color="lightgoldenrodyellow",
            name="Cells",
        )


if __name__ == "__main__":
    main()



import os
import pytest
import shutil

from pathlib import Path

from cellfinder.tools import prep
from imlib.general.exceptions import CommandLineInputError


data_dir = os.path.join("tests", "data")


def test_check_return_ch_ids():
    signal_ch = [0, 1, 3]
    bg_ch = 6
    signal_list = ["file1.txt", "file_2.txt", "file_3.txt"]
    # None given
    assert ([0, 1, 2], 3) == prep.check_and_return_ch_ids(
        None, None, signal_list
    )
    # Only signal given
    assert (signal_ch, 4) == prep.check_and_return_ch_ids(
        signal_ch, None, signal_list
    )

    # Only background given
    assert ([7, 8, 9], bg_ch) == prep.check_and_return_ch_ids(
        None, bg_ch, signal_list
    )

    # Both given (no overlap)
    assert (signal_ch, bg_ch) == prep.check_and_return_ch_ids(
        signal_ch, bg_ch, signal_list
    )

    # Both given (overlap)
    with pytest.raises(CommandLineInputError):
        assert prep.check_and_return_ch_ids(signal_ch, 3, signal_list)


class Args:
    def __init__(
        self,
        model_dir=None,
        empty=None,
        no_detection=False,
        no_classification=False,
        register=False,
        summarise=False,
        no_standard_space=False,
        figures=False,
        output_dir=None,
        registration_output_folder=None,
        cells_file_path=None,
        cubes_output_dir=None,
        classification_out_file=None,
        cells_in_standard_space=None,
        figures_dir=None,
    ):
        self.cell_count_model_dir = model_dir
        self.empty = empty

        self.no_detection = no_detection
        self.no_classification = no_classification
        self.register = register
        self.summarise = summarise
        self.no_standard_space = no_standard_space
        self.figures = figures

        self.output_dir = output_dir

        self.paths = Paths(
            output_dir=registration_output_folder,
            cells_file_path=cells_file_path,
            cubes_output_dir=cubes_output_dir,
            classification_out_file=classification_out_file,
            cells_in_standard_space=cells_in_standard_space,
            figures_dir=figures_dir,
        )


class Paths:
    def __init__(
        self,
        output_dir=None,
        cells_file_path=None,
        cubes_output_dir=None,
        classification_out_file=None,
        cells_in_standard_space=None,
        figures_dir=None,
    ):
        self.registration_output_folder = output_dir
        self.cells_file_path = cells_file_path
        self.tmp__cubes_output_dir = cubes_output_dir
        self.classification_out_file = classification_out_file
        self.cells_in_standard_space = cells_in_standard_space
        self.figures_dir = figures_dir


def get_dict_of_what_to_run(what_to_run):
    what_to_run_dict = {
        "detect": what_to_run.detect,
        "classify": what_to_run.classify,
        "register": what_to_run.register,
        "summarise": what_to_run.summarise,
        "standard_space": what_to_run.standard_space,
        "figures": what_to_run.figures,
    }
    return what_to_run_dict


def test_calc_what_to_run(tmpdir):
    tmpdir = str(tmpdir)

    registered_atlas_path = os.path.join(tmpdir, "registered_atlas.nii")
    cells_file_path = os.path.join(tmpdir, "cells.xml")
    classification_out_file_path = os.path.join(
        tmpdir, "cell_classification.xml"
    )
    cells_in_standard_space_file_path = os.path.join(
        tmpdir, "cells_in_standard_space.xml"
    )
    summary_cell_counts_path = os.path.join(tmpdir, "summary_cell_counts.csv")
    figures_dir = os.path.join(tmpdir, "figures")

    args = Args(
        output_dir=tmpdir,
        registration_output_folder=tmpdir,
        cells_file_path=cells_file_path,
        classification_out_file=classification_out_file_path,
        figures_dir=figures_dir,
        cells_in_standard_space=cells_in_standard_space_file_path,
    )

    # default
    what_to_run = prep.CalcWhatToRun(args)
    default_validation = {
        "detect": True,
        "classify": True,
        "register": False,
        "summarise": False,
        "standard_space": False,
        "figures": False,
    }
    default_test = get_dict_of_what_to_run(what_to_run)
    assert default_validation == default_test

    # options
    args.no_classification = True
    what_to_run = prep.CalcWhatToRun(args)
    validation = {
        "detect": True,
        "classify": False,
        "register": False,
        "summarise": False,
        "standard_space": False,
        "figures": False,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    # hierarchy
    args.summarise = True
    args.figures = True
    what_to_run = prep.CalcWhatToRun(args)
    validation = {
        "detect": True,
        "classify": True,
        "register": True,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    # existance
    Path(registered_atlas_path).touch()
    what_to_run = prep.CalcWhatToRun(args)
    validation = {
        "detect": True,
        "classify": True,
        "register": False,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    Path(cells_file_path).touch()
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": True,
        "register": False,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    os.remove(cells_file_path)
    what_to_run.update(args)
    validation = {
        "detect": True,
        "classify": True,
        "register": False,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    Path(classification_out_file_path).touch()
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": False,
        "register": False,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    Path(summary_cell_counts_path).touch()
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": False,
        "register": False,
        "summarise": False,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    make_and_fill_directory(figures_dir)
    args.figures = True
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": False,
        "register": False,
        "summarise": False,
        "standard_space": True,
        "figures": False,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    Path(cells_in_standard_space_file_path).touch()
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": False,
        "register": False,
        "summarise": False,
        "standard_space": False,
        "figures": False,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test

    os.remove(registered_atlas_path)
    os.remove(summary_cell_counts_path)
    os.remove(cells_in_standard_space_file_path)
    shutil.rmtree(figures_dir)
    what_to_run.update(args)
    validation = {
        "detect": False,
        "classify": False,
        "register": True,
        "summarise": True,
        "standard_space": True,
        "figures": True,
    }
    test = get_dict_of_what_to_run(what_to_run)
    assert validation == test


def make_and_fill_directory(directory):
    os.mkdir(directory)
    for file_size in range(100, 200, 20):
        with open(os.path.join(directory, str(file_size)), "wb") as fout:
            fout.write(os.urandom(file_size))



from datetime import datetime
import logging
import multiprocessing
from multiprocessing import Queue as MultiprocessingQueue
from multiprocessing import Lock

from imlib.general.system import get_sorted_file_paths, get_num_processes

from cellfinder.detect.filters.plane_filters.multiprocessing import (
    MpTileProcessor,
)
from cellfinder.detect.filters.setup_filters import setup_tile_filtering
from cellfinder.detect.filters.volume_filters.multiprocessing import Mp3DFilter


def calculate_parameters_in_pixels(
    x_pixel_um,
    y_pixel_um,
    z_pixel_um,
    soma_diameter_um,
    max_cluster_size_um3,
    ball_xy_size_um,
    ball_z_size_um,
):
    """
    Convert the command-line arguments from real (um) units to pixels
    :param x_pixel_um:
    :param y_pixel_um:
    :param z_pixel_um:
    :param soma_diameter_um:
    :param max_cluster_size_um3:
    :param ball_xy_size_um:
    :param ball_z_size_um:
    :return:
    """

    mean_in_plane_pixel_size = 0.5 * (x_pixel_um + y_pixel_um)
    voxel_volume = x_pixel_um * y_pixel_um * z_pixel_um
    soma_diameter = int(round(soma_diameter_um / mean_in_plane_pixel_size))
    max_cluster_size = int(round(max_cluster_size_um3 / voxel_volume))
    ball_xy_size = int(round(ball_xy_size_um / mean_in_plane_pixel_size))
    ball_z_size = int(round(ball_z_size_um / z_pixel_um))

    return soma_diameter, max_cluster_size, ball_xy_size, ball_z_size


def main(args):
    n_processes = get_num_processes(min_free_cpu_cores=args.n_free_cpus)
    start_time = datetime.now()

    (
        soma_diameter,
        max_cluster_size,
        ball_xy_size,
        ball_z_size,
    ) = calculate_parameters_in_pixels(
        args.x_pixel_um,
        args.y_pixel_um,
        args.z_pixel_um,
        args.soma_diameter,
        args.max_cluster_size,
        args.ball_xy_size,
        args.ball_z_size,
    )

    # file extension only used if a directory is passed
    img_paths = get_sorted_file_paths(
        args.signal_planes_paths[0], file_extension="tif"
    )

    if args.end_plane == -1:
        args.end_plane = len(img_paths)
    planes_paths_range = img_paths[args.start_plane : args.end_plane]

    workers_queue = MultiprocessingQueue(maxsize=n_processes)
    # WARNING: needs to be AT LEAST ball_z_size
    mp_3d_filter_queue = MultiprocessingQueue(maxsize=ball_z_size)
    for plane_id in range(n_processes):
        # place holder for the queue to have the right size on first run
        workers_queue.put(None)

    setup_params = [
        img_paths[0],
        soma_diameter,
        ball_xy_size,
        ball_z_size,
        args.ball_overlap_fraction,
        args.start_plane,
    ]

    mp_3d_filter = Mp3DFilter(
        mp_3d_filter_queue,
        soma_diameter,
        args.output_dir,
        setup_params=setup_params,
        soma_size_spread_factor=args.soma_spread_factor,
        planes_paths_range=planes_paths_range,
        save_planes=args.save_planes,
        plane_directory=args.plane_directory,
        start_plane=args.start_plane,
        max_cluster_size=max_cluster_size,
        outlier_keep=args.outlier_keep,
        artifact_keep=args.artifact_keep,
        save_csv=args.save_csv,
    )

    # fake = Mp3DFilter_fake(setup_params
    #                        )
    # start 3D analysis (waits for planes in queue)
    bf_process = multiprocessing.Process(target=mp_3d_filter.process, args=())
    bf_process.start()  # needs to be started before the loop
    clipping_val, threshold_value = setup_tile_filtering(img_paths[0])
    mp_tile_processor = MpTileProcessor(workers_queue, mp_3d_filter_queue)
    prev_lock = Lock()
    processes = []

    # start 2D tile filter (output goes into queue for 3D analysis)
    for plane_id, path in enumerate(planes_paths_range):
        workers_queue.get()
        lock = Lock()
        lock.acquire()
        p = multiprocessing.Process(
            target=mp_tile_processor.process,
            args=(
                plane_id,
                path,
                prev_lock,
                lock,
                clipping_val,
                threshold_value,
                soma_diameter,
                args.log_sigma_size,
                args.n_sds_above_mean_thresh,
            ),
        )
        prev_lock = lock
        processes.append(p)
        p.start()

    processes[-1].join()
    mp_3d_filter_queue.put((None, None, None))  # Signal the end
    bf_process.join()

    logging.info(
        "Detection complete - all planes done in : {}".format(
            datetime.now() - start_time
        )
    )



import logging

from neuro.heatmap.heatmap import main as heatmap


def figures(args):
    if args.heatmap:
        logging.info("Generating heatmap")
        heatmap(
            args.paths.classification_out_file,
            args.paths.heatmap,
            args.background_planes_path[0],
            args.paths.registered_atlas_path,
            args.heatmap_binning,
            args.x_pixel_um,
            args.y_pixel_um,
            args.z_pixel_um,
            args.heatmap_smooth,
            args.mask_figures,
        )



"""
main
===============

Runs each part of the cellfinder pipeline in turn.

N.B imports are within functions to prevent tensorflow being imported before
it's warnings are silenced
"""

import os
import logging
from datetime import datetime
from imlib.general.logging import suppress_specific_logs

tf_suppress_log_messages = [
    "multiprocessing can interact badly with TensorFlow"
]


def main():
    suppress_tf_logging(tf_suppress_log_messages)
    import amap.main as register

    from cellfinder.tools import prep

    start_time = datetime.now()
    args, what_to_run = prep.prep_cellfinder_general()

    if what_to_run.register:
        # TODO: add register_part_brain option
        logging.info("Registering to atlas")
        args, additional_images_downsample = prep.prep_registration(args)

        register.main(
            args.registration_config,
            args.target_brain_path,
            args.paths.registration_output_folder,
            x_pixel_um=args.x_pixel_um,
            y_pixel_um=args.y_pixel_um,
            z_pixel_um=args.z_pixel_um,
            orientation=args.orientation,
            flip_x=args.flip_x,
            flip_y=args.flip_y,
            flip_z=args.flip_z,
            affine_n_steps=args.affine_n_steps,
            affine_use_n_steps=args.affine_use_n_steps,
            freeform_n_steps=args.freeform_n_steps,
            freeform_use_n_steps=args.freeform_use_n_steps,
            bending_energy_weight=args.bending_energy_weight,
            grid_spacing=args.grid_spacing,
            smoothing_sigma_reference=args.smoothing_sigma_reference,
            smoothing_sigma_floating=args.smoothing_sigma_floating,
            histogram_n_bins_floating=args.histogram_n_bins_floating,
            histogram_n_bins_reference=args.histogram_n_bins_reference,
            sort_input_file=args.sort_input_file,
            n_free_cpus=args.n_free_cpus,
            save_downsampled=not (args.no_save_downsampled),
            additional_images_downsample=additional_images_downsample,
            boundaries=not (args.no_boundaries),
            debug=args.debug,
        )
    else:
        logging.info("Skipping registration")

    if what_to_run.summarise:
        args = prep.prep_atlas_conf(args)

    if len(args.signal_planes_paths) > 1:
        base_directory = args.output_dir

        for idx, signal_paths in enumerate(args.signal_planes_paths):
            channel = args.signal_ch_ids[idx]
            logging.info("Processing channel: " + str(channel))
            channel_directory = os.path.join(
                base_directory, "channel_" + str(channel)
            )
            if not os.path.exists(channel_directory):
                os.makedirs(channel_directory)

            # prep signal channel specific args
            args.signal_planes_paths[0] = signal_paths
            # TODO: don't overwrite args.output_dir - use Paths instead
            args.output_dir = channel_directory
            args.signal_channel = channel
            # Run for each channel
            run_all(args, what_to_run)

    else:
        args.signal_channel = args.signal_ch_ids[0]
        run_all(args, what_to_run)
    logging.info(
        "Finished. Total time taken: {}".format(datetime.now() - start_time)
    )


def run_all(args, what_to_run):
    from cellfinder.detect import detect
    from cellfinder.classify import classify
    import cellfinder.summarise.count_summary as cell_count_summary
    from cellfinder.figures import figures
    from cellfinder.tools import prep
    from cellfinder.standard_space.cells_to_standard_space import (
        transform_cells_to_standard_space,
    )

    args, what_to_run = prep.prep_channel_specific_general(args, what_to_run)

    if what_to_run.detect:
        logging.info("Detecting cell candidates")
        args = prep.prep_candidate_detection(args)
        detect.main(args)
    else:
        logging.info("Skipping cell detection")

    if what_to_run.classify:
        logging.info("Running cell classification")
        args = prep.prep_classification(args)
        classify.main(args)
    else:
        logging.info("Skipping cell classification")

    if what_to_run.summarise:
        logging.info("Summarising cell counts")
        cell_count_summary.analysis_run(args)

    else:
        logging.info("Skipping cell count summary")

    if what_to_run.standard_space:
        logging.info("Converting cells to standard space")
        args = prep.standard_space_prep(args)
        transform_cells_to_standard_space(args)
    else:
        logging.info("Skipping converting cells to standard space")

    if what_to_run.figures:
        logging.info("Generating figures")
        args = prep.figures_prep(args)
        figures.figures(args)
    else:
        logging.info("Skipping figure generation")


def suppress_tf_logging(tf_suppress_log_messages):
    """
    Prevents many lines of logs such as:
    "2019-10-24 16:54:41.363978: I tensorflow/stream_executor/platform/default
    /dso_loader.cc:44] Successfully opened dynamic library libcuda.so.1"
    """
    os.environ["TF_CPP_MIN_LOG_LEVEL"] = "3"

    for message in tf_suppress_log_messages:
        suppress_specific_logs("tensorflow", message)


if __name__ == "__main__":
    main()



import os
import sys
import platform
import pytest
import numpy as np
import pandas as pd

from brainio.brainio import load_nii
from imlib.general.string import get_text_lines

from cellfinder.main import main as cellfinder_run

data_dir = os.path.join(os.getcwd(), "tests", "data", "brain",)


test_output_dir = os.path.join(
    os.getcwd(), "tests", "data", "registration_output", platform.system()
)

x_pix = "40"
y_pix = "40"
z_pix = "50"

relative_tolerance = 0.01
absolute_tolerance = 10
check_less_precise_pd = 1


@pytest.mark.slow
def test_register(tmpdir, test_config_path):
    output_directory = os.path.join(str(tmpdir), "output")
    cellfinder_args = [
        "cellfinder",
        "-s",
        data_dir,
        "-b",
        data_dir,
        "-o",
        output_directory,
        "-x",
        x_pix,
        "-y",
        y_pix,
        "-z",
        z_pix,
        "--n-free-cpus",
        "0",
        "--registration-config",
        test_config_path,
        "--register",
        "--no-detection",
        "--no-classification",
    ]

    sys.argv = cellfinder_args
    cellfinder_run()
    output_directory = os.path.join(output_directory, "registration")

    # a hack because testing on linux on travis is 100% identical to local,
    # but windows is not
    if platform.system() == "Linux":
        image_list = [
            "annotations.nii",
            "boundaries.nii",
            "brain_filtered.nii",
            "control_point_file.nii",
            "downsampled.nii",
            "hemispheres.nii",
            "inverse_control_point_file.nii",
            "registered_atlas.nii",
            "registered_hemispheres.nii",
            "downsampled_channel_0.nii",
        ]
    else:
        image_list = [
            "annotations.nii",
            # "boundaries.nii",
            "brain_filtered.nii",
            "control_point_file.nii",
            "downsampled.nii",
            "hemispheres.nii",
            "inverse_control_point_file.nii",
            # "registered_atlas.nii",
            "registered_hemispheres.nii",
            "downsampled_channel_0.nii",
        ]

    for image in image_list:
        are_images_equal(image, output_directory, test_output_dir)

    assert get_text_lines(
        os.path.join(output_directory, "affine_matrix.txt")
    ) == get_text_lines(os.path.join(test_output_dir, "affine_matrix.txt"))

    assert get_text_lines(
        os.path.join(output_directory, "invert_affine_matrix.txt")
    ) == get_text_lines(
        os.path.join(test_output_dir, "invert_affine_matrix.txt")
    )

    pd.testing.assert_frame_equal(
        pd.read_csv(os.path.join(output_directory, "volumes.csv")),
        pd.read_csv(os.path.join(test_output_dir, "volumes.csv")),
        check_exact=False,
        check_less_precise=check_less_precise_pd,
    )


def are_images_equal(image_name, output_directory, test_output_directory):
    image = load_nii(
        os.path.join(output_directory, image_name),
        as_array=True,
        as_numpy=True,
    )
    test_image = load_nii(
        os.path.join(test_output_directory, image_name),
        as_array=True,
        as_numpy=True,
    )
    np.testing.assert_allclose(
        image, test_image, rtol=relative_tolerance, atol=absolute_tolerance
    )



import os
import sys
import pytest

from math import isclose

from cellfinder.main import main as cellfinder_run
import imlib.IO.cells as cell_io

data_dir = os.path.join(
    os.getcwd(), "tests", "data", "integration", "detection"
)
signal_data = os.path.join(data_dir, "crop_planes", "ch0")
background_data = os.path.join(data_dir, "crop_planes", "ch1")
cells_validation_xml = os.path.join(data_dir, "cell_classification.xml")

x_pix = "2"
y_pix = "2"
z_pix = "5"

DETECTION_TOLERANCE = 2


# FIXME: This isn't a very good example


@pytest.mark.slow
def test_detection_full(tmpdir):
    tmpdir = str(tmpdir)

    cellfinder_args = [
        "cellfinder",
        "-s",
        signal_data,
        "-b",
        background_data,
        "-o",
        tmpdir,
        "-x",
        x_pix,
        "-y",
        y_pix,
        "-z",
        z_pix,
        "--n-free-cpus",
        "0",
        "--no-standard-space",
        "--save-planes",
    ]
    sys.argv = cellfinder_args
    cellfinder_run()

    cells_test_xml = os.path.join(tmpdir, "cell_classification.xml")

    cells_validation = cell_io.get_cells(cells_validation_xml)
    cells_test = cell_io.get_cells(cells_test_xml)

    num_non_cells_validation = sum(
        [cell.type == 1 for cell in cells_validation]
    )
    num_cells_validation = sum([cell.type == 2 for cell in cells_validation])

    num_non_cells_test = sum([cell.type == 1 for cell in cells_test])
    num_cells_test = sum([cell.type == 2 for cell in cells_test])

    assert isclose(
        num_non_cells_validation,
        num_non_cells_test,
        abs_tol=DETECTION_TOLERANCE,
    )
    assert isclose(
        num_cells_validation, num_cells_test, abs_tol=DETECTION_TOLERANCE
    )



import logging

from argparse import ArgumentParser, ArgumentDefaultsHelpFormatter
from datetime import datetime
from os.path import join

from imlib.general.system import (
    ensure_directory_exists,
    safe_execute_command,
    SafeExecuteCommandError,
)
from imlib.IO.cells import get_cells, save_cells
from imlib.cells.cells import transform_cell_positions
from imlib.general.system import delete_temp
from imlib.general.exceptions import TransformationError
from imlib.image.metadata import define_pixel_sizes
from imlib.source.source_files import source_custom_config_cellfinder

from fancylog import fancylog

from brainio.brainio import load_any as load_any_image
from amap.register.registration_params import RegistrationParams
from amap.cli import registration_parse, geometry_parser

from cellfinder.tools import prep
import cellfinder.tools.parser as cellfinder_parse

import cellfinder as program_name


def transform_cells_to_standard_space(args):
    if args.registration_config is None:
        args.registration_config = source_custom_config_cellfinder()

    reg_params = RegistrationParams(
        args.registration_config,
        affine_n_steps=args.affine_n_steps,
        affine_use_n_steps=args.affine_use_n_steps,
        freeform_n_steps=args.freeform_n_steps,
        freeform_use_n_steps=args.freeform_use_n_steps,
        bending_energy_weight=args.bending_energy_weight,
        grid_spacing=args.grid_spacing,
        smoothing_sigma_reference=args.smoothing_sigma_reference,
        smoothing_sigma_floating=args.smoothing_sigma_floating,
        histogram_n_bins_floating=args.histogram_n_bins_floating,
        histogram_n_bins_reference=args.histogram_n_bins_reference,
    )

    generate_deformation_field(args, reg_params)
    cells_only = not args.transform_all
    cells = get_cells(
        args.paths.classification_out_file, cells_only=cells_only
    )

    logging.info("Loading deformation field")
    deformation_field = load_any_image(
        args.paths.tmp__deformation_field, as_numpy=True
    )
    scales = get_scales(args, reg_params)
    field_scales = get_deformation_field_scales(reg_params)

    logging.info("Transforming cell positions")
    transformed_cells = transform_cell_positions(
        cells, deformation_field, field_scales, scales
    )

    logging.info("Saving transformed cell positions")

    save_cells(
        transformed_cells,
        args.paths.cells_in_standard_space,
        save_csv=args.save_csv,
    )

    if not args.debug:
        logging.info("Removing standard space transformation temp files")
        delete_temp(args.paths.standard_space_output_folder, args.paths)


def get_deformation_field_scales(reg_params):
    """
    Calculates the scaling of the deformation field from real space
    to voxels
    :param reg_params:
    :return:
    """
    x_scale = 1000 / float(reg_params.atlas_x_pix_size)
    y_scale = 1000 / float(reg_params.atlas_y_pix_size)
    z_scale = 1000 / float(reg_params.atlas_z_pix_size)

    return x_scale, y_scale, z_scale


def get_scales(args, reg_params):
    """
    Calculate the scaling factors from the coordinate space of the raw data
    (which the cell positions are defined in) to the (usually downsampled)
    reference space that the deformation field is defined in.
    :param args:
    :param reg_params:
    :return:
    """
    x_scale = args.x_pixel_um / float(reg_params.atlas_x_pix_size)
    y_scale = args.y_pixel_um / float(reg_params.atlas_y_pix_size)
    z_scale = args.z_pixel_um / float(reg_params.atlas_z_pix_size)

    return x_scale, y_scale, z_scale


def prepare_deformation_field_cmd(args, reg_params):
    """
    Prepares the niftyreg command to generate the deformation field from the
    control point file (allready generated in registration).
    :param args:
    :param reg_params:
    :return:
    """
    cmd = "{} -def {} {} -ref {}".format(
        reg_params.transform_program_path,
        args.paths.control_point_file_path,
        args.paths.tmp__deformation_field,
        args.paths.downsampled_background,
    )
    return cmd


def generate_deformation_field(args, reg_params):
    logging.info("Generating deformation field")
    try:
        safe_execute_command(
            prepare_deformation_field_cmd(args, reg_params),
            args.paths.tmp__deformation_log_file_path,
            args.paths.tmp__deformation_error_path,
        )
    except SafeExecuteCommandError as err:
        raise TransformationError(
            "Generation of deformation field failed ; {}".format(err)
        )


def cells_standard_space_cli_parser():
    parser = ArgumentParser(formatter_class=ArgumentDefaultsHelpFormatter)
    parser = cli_parse(parser)
    parser = cellfinder_parse.pixel_parser(parser)
    parser = cellfinder_parse.standard_space_parse(parser)
    parser = cellfinder_parse.misc_parse(parser)
    parser = geometry_parser(parser)
    parser = registration_parse(parser)

    return parser


def cli_parse(parser):
    cli_parser = parser.add_argument_group("Command line options")
    cli_parser.add_argument(
        "--cells",
        dest="cells_file_path",
        type=str,
        required=True,
        help="Path of the xml file containing cells to be transformed",
    )

    cli_parser.add_argument(
        "--transformation",
        dest="transformation",
        type=str,
        required=True,
        help="Path to the control point file (from niftyreg or cellfinder "
        "registration).",
    )

    cli_parser.add_argument(
        "-o",
        "--output-dir",
        dest="output_dir",
        type=str,
        required=True,
        help="Directory to save the cubes into",
    )

    cli_parser.add_argument(
        "--ref",
        dest="reference",
        type=str,
        required=True,
        help="Reference nii image, in the same space as the downsampled raw "
        "data.",
    )

    return parser


def cli_path_update(paths, args):
    paths.control_point_file_path = args.transformation
    paths.downsampled_background = args.reference
    paths.classification_out_file = args.cells_file_path
    paths.output_dir = args.output_dir


def main():
    start_time = datetime.now()
    args = cells_standard_space_cli_parser().parse_args()
    args.paths = prep.Paths(args.output_dir)
    args.paths.standard_space_output_folder = args.output_dir
    args.paths.cells_in_standard_space = join(
        args.paths.output_dir, "cells_in_standard_space.xml"
    )
    cli_path_update(args.paths, args)
    args.paths.make_invert_cell_position_paths()
    args = define_pixel_sizes(args)

    # TODO: implement a recursive function to remove the need to do this
    # (probably using pathlib)
    ensure_directory_exists(args.paths.output_dir)
    ensure_directory_exists(args.paths.standard_space_output_folder)
    fancylog.start_logging(
        args.paths.output_dir,
        log_to_file=False,
        package=program_name,
        variables=args,
        verbose=args.debug,
        filename="cells_to_standard_space",
        log_header="CELL TRANSFORMATION TO STANDARD SPACE LOG",
    )
    logging.info("Starting transformation of cell positions")
    transform_cells_to_standard_space(args)
    logging.info("Finished. Total time taken: %s", datetime.now() - start_time)


if __name__ == "__main__":
    main()



import platform
from setuptools import setup, find_namespace_packages, Extension
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

requirements = [
    "numpy",
    "scikit-learn",
    "configparser",
    "pandas<=0.25.3,>=0.25.1",
    "packaging",
    "scikit-image",
    "tifffile",
    "natsort",
    "tqdm",
    "anytree",
    "h5py",
    "multiprocessing-logging",
    "psutil",
    "nibabel",
    "configobj",
    "scipy>=0.18",
    "toolz>=0.7.3",
    "tensorflow>=2.2.0",
    "dask >= 2.15.0",
    "napari>=0.3.0",
    "slurmio>=0.0.4",
    "brainio>=0.0.15",
    "fancylog>=0.0.7",
    "micrometa>=0.0.11",
    "imlib>=0.0.25",
    "neuro>=0.0.12",
    "amap>=0.1.22",
]


if platform.system() == "Windows":
    # FIXME: There must be a better way of doing this.
    base_tile_filter_extension = Extension(
        name="cellfinder.detect.filters.plane_filters.base_tile_filter",
        sources=[
            "cellfinder/detect/filters/plane_filters/" "base_tile_filter.pyx"
        ],
        language="c++",
    )

    ball_filter_extension = Extension(
        name="cellfinder.detect.filters.volume_filters.ball_filter",
        sources=[
            "cellfinder/detect/filters/volume_filters/" "ball_filter.pyx"
        ],
    )

    structure_detection_extension = Extension(
        name="cellfinder.detect.filters.volume_filters.structure_detection",
        sources=[
            "cellfinder/detect/filters/volume_filters/"
            "structure_detection.pyx"
        ],
        language="c++",
    )
else:
    base_tile_filter_extension = Extension(
        name="cellfinder.detect.filters.plane_filters.base_tile_filter",
        sources=[
            "cellfinder/detect/filters/plane_filters/" "base_tile_filter.pyx"
        ],
        libraries=["m"],
        language="c++",
    )

    ball_filter_extension = Extension(
        name="cellfinder.detect.filters.volume_filters.ball_filter",
        sources=[
            "cellfinder/detect/filters/volume_filters/" "ball_filter.pyx"
        ],
        libraries=["m"],
    )

    structure_detection_extension = Extension(
        name="cellfinder.detect.filters.volume_filters.structure_detection",
        sources=[
            "cellfinder/detect/filters/volume_filters/"
            "structure_detection.pyx"
        ],
        libraries=["m"],
        language="c++",
    )


setup(
    name="cellfinder",
    version="0.3.14rc1",
    description="Automated 3D cell detection and registration of whole-brain images",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    extras_require={
        "dev": [
            "sphinx",
            "recommonmark",
            "sphinx_rtd_theme",
            "pydoc-markdown",
            "black",
            "pytest-cov",
            "pytest",
            "gitpython",
            "coverage>=5.0.3",
        ]
    },
    setup_requires=["cython"],
    python_requires=">=3.6, <3.8",
    packages=find_namespace_packages(exclude=("docs", "doc_build", "tests")),
    include_package_data=True,
    ext_modules=[
        ball_filter_extension,
        structure_detection_extension,
        base_tile_filter_extension,
    ],
    entry_points={
        "console_scripts": [
            "cellfinder = cellfinder.main:main",
            "cellfinder_download = cellfinder.download.cli:main",
            "cellfinder_train = cellfinder.train.train_yml:main",
            "cellfinder_view_cells = cellfinder.viewer.two_dimensional:main",
            "cellfinder_count_summary = "
            "cellfinder.summarise.count_summary:main",
            "cellfinder_region_summary = "
            "cellfinder.analyse.group.region_summary:main",
            "cellfinder_xml_crop = cellfinder.utils.xml_crop:main",
            "cellfinder_xml_scale = cellfinder.utils.xml_scale:main",
            "cellfinder_cell_standard = "
            "cellfinder.standard_space.cells_to_standard_space:main",
            "cellfinder_gen_region_vol = "
            "cellfinder.utils.generate_region_volume:main",
            "cellfinder_cells_to_brainrender = "
            "neuro.points.points_to_brainrender:main",
            "cellfinder_curate = cellfinder.train.curation:main",
        ]
    },
    url="https://cellfinder.info",
    project_urls={
        "Source Code": "https://github.com/SainsburyWellcomeCentre/cellfinder",
        "Bug Tracker": "https://github.com/SainsburyWellcomeCentre/cellfinder/issues",
        "Documentation": "https://docs.cellfinder.info",
    },
    author="Adam Tyson, Christian Niedworok, Charly Rousseau",
    author_email="adam.tyson@ucl.ac.uk",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Operating System :: POSIX :: Linux",
        "Operating System :: Microsoft :: Windows :: Windows 10",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Scientific/Engineering :: Image Recognition",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
    ],
    zip_safe=False,
)



import numpy as np

import cellfinder.tools.geometry as geometry


def test_make_sphere():
    pass


def test_four_connected_kernel():
    assert (
        (np.array([[0, 1, 0], [1, 1, 1], [0, 1, 0]], dtype=np.bool))
        == geometry.four_connected_kernel()
    ).all()



import logging
import tensorflow as tf


def allow_gpu_memory_growth():
    """
    If a gpu is present, prevent tensorflow from using all the memory straight
    away. Allows multiple processes to use the GPU (and avoid occasional
    errors on some systems) at the cost of a slight performance penalty.
    """
    gpus = tf.config.experimental.list_physical_devices("GPU")
    if gpus:
        logging.debug("Allowing GPU memory growth")
        try:
            # Currently, memory growth needs to be the same across GPUs
            for gpu in gpus:
                tf.config.experimental.set_memory_growth(gpu, True)
            logical_gpus = tf.config.experimental.list_logical_devices("GPU")
            logging.debug(
                f"{len(gpus)} physical GPUs, {len(logical_gpus)} logical GPUs"
            )
        except RuntimeError as e:
            # Memory growth must be set before GPUs have been initialized
            print(e)
    else:
        logging.debug("No GPUs found, using CPU.")


def set_tf_threads(max_threads):
    """
    Limit the number of threads that tensorflow uses
    :param max_threads: Maximum number of threads to use
    :return:
    """
    logging.debug(
        f"Setting maximum number of threads for tensorflow "
        f"to: {max_threads}"
    )

    # If statements are for testing. If tf is initialised, then setting these
    # parameters throws an error
    if tf.config.threading.get_inter_op_parallelism_threads() != 0:
        tf.config.threading.set_inter_op_parallelism_threads(max_threads)
    if tf.config.threading.get_intra_op_parallelism_threads() != 0:
        tf.config.threading.set_intra_op_parallelism_threads(max_threads)



import os
import sys
import pytest

from imlib.general.config import get_config_obj
from cellfinder.download.cli import main as cellfinder_download

data_dir = os.path.join(os.getcwd(), "tests", "data", "brain",)
test_output_dir = os.path.join(
    os.getcwd(), "tests", "data", "registration_output",
)

TEST_ATLAS = "allen_2017_100um"


def download_atlas(directory):
    download_args = [
        "cellfinder_download",
        "--atlas",
        TEST_ATLAS,
        "--install-path",
        directory,
        "--no-amend-config",
        "--no-models",
    ]
    sys.argv = download_args
    cellfinder_download()
    return directory


def generate_test_config(atlas_dir):
    config = os.path.join(os.getcwd(), "tests", "data", "config", "test.conf")
    config_obj = get_config_obj(config)
    atlas_conf = config_obj["atlas"]
    orig_base_directory = atlas_conf["base_folder"]

    with open(config, "r") as in_conf:
        data = in_conf.readlines()
    for i, line in enumerate(data):
        data[i] = line.replace(
            f"base_folder = '{orig_base_directory}",
            f"base_folder = '{os.path.join(atlas_dir, 'atlas', TEST_ATLAS)}",
        )
    test_config = os.path.join(atlas_dir, "config.conf")
    with open(test_config, "w") as out_conf:
        out_conf.writelines(data)

    return test_config


@pytest.fixture()
def test_config_path(tmpdir):
    print("fixture")
    atlas_directory = str(tmpdir)
    download_atlas(atlas_directory)
    test_config = generate_test_config(atlas_directory)
    return test_config
